﻿// Copyright 2009-2011 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

// The function outputList shows all the values in a list
let outputList l = 
    for e in l do
        printf "%A " e
    printfn ""

// The function mapList iterates recursively over a list
// and calls the function f for each of the elements, then
// returns a new list constructed out of all the return values.
let rec mapList f (l: 'a list) = 
    match l with
    | [] -> []
    | hd :: tl -> f hd :: (mapList f tl)

// This call to mapList calculates the squares of all the list values
mapList (fun x -> x * x) [1;2;3;4;5] |> outputList

// using Partial Application to create a new function
// squareList which takes an int list as a parameter and returns
// an int list with all the square values

// squareList is constructed from mapList - it will
// square all the values in a given list
let squareList = mapList (fun x -> x * x)

// based on the implementation of mapList above,
// here's the function filterList. This iterates over the 
// values in a given list and applies to each one a given function.
// It returns a new list consisting of only those
// results for which the given function returned true.
// As an example, this call:
// filterList (fun x -> x > 3) [1;2;3;4;5;6;7]
// returns this list: [4;5;6;7]

// filterList uses a recursive approach to iterate over 
// a list, applies the given function to each element and 
// constructs a result list that contains only those elements
// of the original list for which the function returned true.
let rec filterList f (l: 'a list) = 
    match l with
    | [] -> []
    | hd :: tl -> 
        (if f hd then [hd] else []) @ (filterList f tl)

// Using partial application to construct a new function
// filterEvens from filterList, to filter a given int list
// for even values.

// using Partial Application, filterEvens is constructed from
// filterList to apply a filter for even numbers
let filterEvens = filterList (fun x -> x % 2 = 0)

// Using composition to construct a new function out of 
// the ones we have created so far. This new function takes 
// an int list as a parameter, squares all the values in the list,
// filters them to return only the even results and outputs those.
// As an example, this call:
// outputEvenSquares [1;2;3;4;5;6;7;8;9;10]
// should output 4 16 36 64 100

// using Composition, a new function is constructed out of building
// blocks to show the even squares based on a list of ints
let outputEvenSquares = squareList >> filterEvens >> outputList

// this will show 4 16 36 64 100 - filtering out 1, 9, 25, 49 and 81
outputEvenSquares [1;2;3;4;5;6;7;8;9;10]

