﻿// Copyright 2009-2011 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

open System.Collections.Generic

// this is the memoize function again
let memoize (f: 'a -> 'b) =
    let cache = new Dictionary<'a, 'b>()
    fun v ->
        if not (cache.ContainsKey(v)) then 
            cache.Add(v, f(v))
        cache.[v]
        
// a function memoizeMax that takes
// a max parameter in addition to a function, and restricts
// the maximum number of memoized return values to this 
// given maximum, removing the oldest ones when the 
// result memory grows too large.

// this implementation uses a queue to store the key values
// this makes it easy to pop off the oldest ones when needed
let memoizeMax max (f: 'a -> 'b) = 
    let cache = new Dictionary<'a, 'b>()
    let queue = new Queue<'a>()
    fun v ->
        if not (cache.ContainsKey(v)) then
            printfn "Calculating and caching value for %A" v
            cache.Add(v, f(v))
            queue.Enqueue v
            if queue.Count > max then
                let oldValue = queue.Dequeue()
                printfn "Queue is full, dropping value for %A" oldValue
                cache.Remove(oldValue) |> ignore
        else
            printfn "Returning result for %A from memory" v
        cache.[v]

let square x = x * x
let memoizedSquare = memoizeMax 2 square
let outputResult = printfn "%d"

memoizedSquare 3 |> outputResult
memoizedSquare 4 |> outputResult
memoizedSquare 3 |> outputResult
memoizedSquare 4 |> outputResult
memoizedSquare 5 |> outputResult
memoizedSquare 3 |> outputResult
memoizedSquare 3 |> outputResult

// A main consideration for parameter order is the use of the function with 
// partial application, composition and related mechanisms 
// like the use of the |> and >> operators in F#.
// To make best use of these mechanisms, the parameter
// order has to match the use cases.

// this function has the parameters turned around
let memoizeMaxChanged f m = memoizeMax m f
